#include <XSUtil/FunctionUtils/FunctionUtility.h>
#include <XSUtil/FunctionUtils/xsFortran.h>
#include <XSUtil/Utils/XSutility.h>
#include <sstream>


typedef XSutility::auto_array_ptr<float> FloatAP;
typedef XSutility::auto_array_ptr<int> IntAP;


extern "C" void xneispec_(int& ntemp, float* temp, int& nion, float* ionfrac, int* ionel, 
                   int* ionstage, int& ne, int* ear, float& z, float* linesh, int& nabun, 
                   float* abund, int* abunel, int& qcalc, float* photar, int& nmtval, 
                   float* tval, int& nmions, float* cont, float* cenerg, int* conEl, 
                   int* conIon, int* conpt, int* consiz, float* pseudo, float* penerg,
                   int* psupt, int* psusiz, float* lineE, float* lineEm, int* lineEl,
                   int* lineIon, int* linpt, int* linsiz, int& qtherm, float& velocity, int& status);
         


void ldneiaped(FloatAP& tval, long& nmtval, long& nmions, FloatAP& cont, FloatAP& cenerg,
        IntAP& conEl, IntAP& conIon, IntAP& conpt, IntAP& consiz, FloatAP& pseudo,
        FloatAP& penerg, IntAP& psupt, IntAP& psusiz, FloatAP& lineE, FloatAP& lineEm,
        IntAP& lineEl, IntAP& lineIon, IntAP& linpt, IntAP& linsiz, int& status);

extern "C" void nneispec_(int& ntemp, float* temp, int& nion, float* ionfrac, int* ionel, 
        int* ionstage, int& ne, int* ear, float& z, float* linesh, int& nabun,
         float* abund, int* abunel, int& qcalc, float* photar)
{
   // Translated from subroutine formerly part of neispec.f by C.Gordon 01/09
   
   // Routine to return the spectrum from a multi-temperature NEI plasma. 
   // This uses the APEC tables. This routine loads the APEC data arrays
   // then calls xneispec to actually do the work.

   // Arguments
   //    temp         r        i: temperature (keV)
   //    nion         i        i: number of different ions
   //    ionfrac      r        i: fraction in ion
   //    ionel        i        i: element for this ion
   //    ionstage     i        i: level for this ion (ionel+1 = fully stripped)
   //    ne           i        i: number of response energies
   //    ear          r        i: response energies
   //    z            r        i: redshift
   //    linesh       r        i: lineshape parameters
   //    nabun        i        i: number of elements with abundances
   //    abund        r        i: abundances
   //    abunel       i        i: element for this abundance
   //    qcalc        l        i: if true force a recalculation of all arrays
   //    photar       r        r: output spectrum

   // Pointers to local arrays
   //    apTval        Tabulated temperatures
   //    apCont        Tabulated continua
   //    apCenerg      Energies for tabulated continua
   //    apConEl       Atomic numbers for tabulated continua
   //    apConIon      Ion stages for tabulated continua
   //    apConpt       Offsets in the cont array for each ion and kT
   //    apConsiz      Sizes in the cont array for each ion and kT
   //    apPseudo      Tabulated pseudo-continua
   //    apPenerg      Energies for tabulated pseudo-continua
   //    apPsupt       Offsets in the pseudo array for each ion and kT
   //    apPsusiz      Sizes in the pseudo array for each ion and kT
   //    apLineE       Tabulated line energies
   //    apLineEm      Tabulated line emissivities
   //    apLineEl      Tabulated line element (by atomic number)
   //    apLineIon     Tabulated line ion
   //    apLinpt       Offsets in the line arrays for each kT
   //    apLinsiz      Sizes in the line arrays for each kT

   //    nmtval       The number of tabulated temperatures
   //    nmions       The number of tabulated ions

   static long nmtval=0;
   static long nmions=0;
   static bool qfirst=true;
   static string apecroot;
   static FloatAP apTval(0);
   static FloatAP apCont(0);
   static FloatAP apCenerg(0);
   static IntAP apConEl(0);
   static IntAP apConIon(0);
   static IntAP apConpt(0);
   static IntAP apConsiz(0);
   static FloatAP apPseudo(0);
   static FloatAP apPenerg(0);
   static IntAP apPsupt(0);
   static IntAP apPsusiz(0);
   static FloatAP apLineE(0);
   static FloatAP apLineEm(0);
   static IntAP apLineEl(0);
   static IntAP apLineIon(0);
   static IntAP apLinpt(0);
   static IntAP apLinsiz(0);
   
   int status=0;
   
   // If the user has switched NEI APEC files then we reset qfirst
   
   string pname("NEIAPECROOT");
   string pvalue(FunctionUtility::getModelString(pname));
   if (pvalue != apecroot)
   {
      qfirst = true;
      apecroot = pvalue;
   }

   // If the first time through then load the model file. 

   if (qfirst)
   {
      ldneiaped(apTval, nmtval, nmions, apCont, apCenerg, apConEl, apConIon,
            apConpt, apConsiz, apPseudo, apPenerg, apPsupt, apPsusiz, 
            apLineE, apLineEm, apLineEl, apLineIon, apLinpt, apLinsiz, status);
      qfirst = (status != 0);
   }
   
   if (status)
   {
      std::ostringstream oss;
      oss << "Error " << status << " in NNEISPEC\nFailure in LDNEIAPED";
      string msg(oss.str());
      xs_write(const_cast<char*>(msg.c_str()),10);
   }
   else
   {
      // Check for values of APECTHERMAL and APECVELOCITY xset parameters
      // Check the APECTHERMAL variable - if the variable has not been set then
      // assume "false"

      pname = "APECTHERMAL";
      pvalue = FunctionUtility::getModelString(pname);
      int qtherm = 0;
      if (pvalue.length() && pvalue != FunctionUtility::NOT_A_KEY())
      {
         pvalue = XSutility::lowerCase(pvalue);
         if (pvalue.find("yes") == 0 || pvalue.find("on") == 0)
            qtherm = 1;
         else if (pvalue.find("no") == 0 || pvalue.find("off") == 0)
            qtherm = 0;
      }

      // and the APECVELOCITY variable - if it doesn't exist then use 0.

      pname = "APECVELOCITY";
      pvalue = FunctionUtility::getModelString(pname);
      float velocity = 0.0;
      if (pvalue.length() && pvalue != FunctionUtility::NOT_A_KEY())
      {
         std::istringstream iss(pvalue);
         float tmpVel = 0.0;
         if (!(iss >> tmpVel) || !iss.eof())
         {
            string msg("Failed to read value from APECVELOCITY - assuming 0.");
            xs_write(const_cast<char*>(msg.c_str()),10);
            velocity = 0.0;
         }
         else
            velocity = tmpVel;
      }
      
      string msg1("Thermal broadening : ");
      qtherm ? msg1 += "T" : msg1 += "F";
      xs_write(const_cast<char*>(msg1.c_str()), 25);
      std::ostringstream oss1;
      oss1 << "Velocity broadening : " << velocity;
      xs_write(const_cast<char*>(oss1.str().c_str()), 25);
      
      // Call the routine to do the work
      
      int intnmtval = static_cast<int>(nmtval);
      int intnmions = static_cast<int>(nmions);
      xneispec_(ntemp, temp, nion, ionfrac, ionel, ionstage, ne, ear, z,
         linesh, nabun, abund, abunel, qcalc, photar, intnmtval, apTval.get(),
         intnmions, apCont.get(), apCenerg.get(), apConEl.get(), apConIon.get(),
         apConpt.get(), apConsiz.get(), apPseudo.get(), apPenerg.get(),
         apPsupt.get(), apPsusiz.get(), apLineE.get(), apLineEm.get(),
         apLineEl.get(), apLineIon.get(), apLinpt.get(), apLinsiz.get(),
         qtherm, velocity, status);
         
      if (status)
      {
         std::ostringstream oss;
         oss << "Error " << status << " in NNEISPEC\nFailure in XNEISPEC";
         xs_write(const_cast<char*>(oss.str().c_str()), 10);
      }
   }
}
